"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var shallowEqual_1 = __importDefault(require("fbjs/lib/shallowEqual"));
var rxjs_1 = require("rxjs");
var operators_1 = require("rxjs/operators");
var stapp_1 = require("stapp");
var identity_1 = require("stapp/lib/helpers/identity/identity");
var logError_1 = require("stapp/lib/helpers/logError/logError");
var constants_1 = require("./constants");
var helpers_1 = require("./helpers");
/**
 * Persistence module.
 * @typeparam State Application state shape
 * @param config Configuration object
 * @param config.key Key, used to store data in a storage.
 * @param config.storage Storage to use.
 * @returns Module
 */
exports.persist = function (_a) {
    var key = _a.key, storage = _a.storage, blackList = _a.blackList, whiteList = _a.whiteList, _b = _a.transforms, transforms = _b === void 0 ? [] : _b, _c = _a.throttle, throttle = _c === void 0 ? 0 : _c, _serialize = _a.serialize, stateReconciler = _a.stateReconciler, _d = _a.timeout, _timeout = _d === void 0 ? constants_1.DEFAULT_TIMEOUT : _d;
    var storageKey = constants_1.PERSIST + ":" + key;
    var serialize = _serialize === false ? identity_1.identity : helpers_1.defaultSerialize;
    var deserialize = _serialize === false ? identity_1.identity : helpers_1.defaultDeserialize;
    var merge = stateReconciler || helpers_1.defaultMerge;
    var clearStorage = function () {
        return storage.removeItem(storageKey);
    };
    var loaded = false;
    var setEpic = function (_, state$) {
        return state$.pipe(operators_1.filter(function () { return loaded; }), throttle > 0 ? operators_1.auditTime(throttle) : identity_1.identity, operators_1.map(helpers_1.pick(blackList, whiteList)), operators_1.distinctUntilChanged(shallowEqual_1.default), operators_1.map(helpers_1.mapTransform(transforms)), operators_1.tap(function (state) {
            storage
                .setItem(storageKey, serialize(state))
                .catch(function (error) { return logError_1.logError(constants_1.PERSIST, error); });
        }), operators_1.map(function () { return null; }));
    };
    var getEpic = function (_, state$, _a) {
        var getState = _a.getState;
        return rxjs_1.defer(function () { return storage.getItem(storageKey); }).pipe(operators_1.tap(function () { return (loaded = true); }), operators_1.timeout(_timeout), operators_1.filter(Boolean), operators_1.map(deserialize), operators_1.map(helpers_1.mapTransformRight(transforms)), operators_1.map(helpers_1.pick(blackList, whiteList)), operators_1.map(function (restoredState) { return merge(restoredState, getState()); }), operators_1.map(stapp_1.dangerouslyReplaceState), operators_1.catchError(function (error) {
            logError_1.logError(constants_1.PERSIST, error);
            return rxjs_1.EMPTY;
        }));
    };
    return {
        name: constants_1.PERSIST,
        api: {
            clearStorage: clearStorage
        },
        epic: [setEpic, getEpic],
        useGlobalObservableConfig: false
    };
};
//# sourceMappingURL=persist.js.map